@@ -49,6 +49,7 @@ import functools
 import contextlib
 from enum import Enum
 from types import SimpleNamespace
+from functools import wraps
 from collections import namedtuple, defaultdict
 from collections.abc import Sequence, Mapping, MutableSequence
 
@@ -59,8 +60,8 @@ try:
     from PySide2.QtWidgets import QMessageBox, QListWidget, QAbstractItemView
     from PySide2.QtWidgets import QPlainTextEdit, QHBoxLayout, QComboBox
     from PySide2.QtWidgets import QSplashScreen, QFileDialog, QButtonGroup
-    from PySide2.QtWidgets import QProgressBar
-    from PySide2.QtGui import QPixmap, QIcon
+    from PySide2.QtWidgets import QProgressBar, QGroupBox
+    from PySide2.QtGui import QPixmap, QIcon, QFont
     from PySide2.QtCore import Qt, QTimer, Signal, QEvent
 except ImportError:
     try:
@@ -70,12 +71,12 @@ except ImportError:
         from PyQt5.QtWidgets import QMessageBox, QListWidget, QAbstractItemView
         from PyQt5.QtWidgets import QPlainTextEdit, QHBoxLayout, QComboBox
         from PyQt5.QtWidgets import QSplashScreen, QFileDialog, QButtonGroup
-        from PyQt5.QtWidgets import QProgressBar
-        from PyQt5.QtGui import QPixmap, QIcon
+        from PyQt5.QtWidgets import QProgressBar, QGroupBox
+        from PyQt5.QtGui import QPixmap, QIcon, QFont
         from PyQt5.QtCore import Qt, QTimer, QEvent
         from PyQt5.QtCore import pyqtSignal as Signal
-    except ImportError:
-        raise Exception('At least one of PySide2 or PyQt5 must be installed')
+    except ImportError as e:
+        raise Exception('At least one of PySide2 or PyQt5 must be installed') from e
 
 # We need a QApplication before creating any widgets
 if QApplication.instance() is None:
@@ -89,6 +90,7 @@ signal.signal(signal.SIGINT, signal.SIG_DFL)
 E = QLineEdit
 C = QCheckBox
 R = QRadioButton
+G = QGroupBox
 
 
 class III:
@@ -145,8 +147,10 @@ class ContextMixIn():
                     break
                 idx -= 1
         else:
-            lines, first = inspect.getsourcelines(self._start.frame)
-            withlines = lines[self._start.lineno + first -1 : end.lineno + first]
+            # Do not use inspect.getsourcelines because it appears
+            # to fail on some systems
+            lines = open(self._start.filename).readlines()
+            withlines = lines[self._start.lineno - 1 : end.lineno]
 
         withsource = textwrap.dedent(''.join(withlines))
 
@@ -175,7 +179,7 @@ class ContextMixIn():
         else:
             widget = self
 
-        _connect(None, widget, signal_name='default', slot=locals_copy['slot'])
+        connect(widget, slot=locals_copy['slot'])
 
         return True   # Cancel the exception raised by the first execution
 
@@ -241,7 +245,13 @@ class _Analyzer(ast.NodeVisitor):
 # We cannot use real properties because they are defined on the class
 # and would be shared by different GUIs!
 
-_InstanceProperty = namedtuple('InstanceProperty', 'get set')
+GuiettaProperty = namedtuple('GuiettaProperty', 'get set')
+GuiettaProperty.__doc__ = '''
+Holds the get/set methods for a Guietta magic property.
+
+get() - returns the property value
+set(x) - sets the property to x
+'''
 
 
 class _ContextStr(str, ContextMixIn):
@@ -274,6 +284,53 @@ class _ContextDict(dict, ContextMixIn):
         self._widget = widget
 
 
+def _alsoAcceptAnotherGui(widget):
+    '''Decorator for the set() function of a guietta property.
+
+    Adds the possibility of receiving a Gui instance as the value,
+    replacing the widget with the Gui widget. In case of the widget
+    is a QGroupBox instance, the GUi is placed as the inner layout instead.
+    '''
+    def decorator(f):
+        @wraps(f)
+        def wrapper(value):
+            if isinstance(value, Gui):
+                if isinstance(widget, QGroupBox):
+                    widget.setLayout(value.layout())
+                else:
+                    widget._gui.layout().replaceWidget(widget, value.window())
+                    widget.hide()
+            else:
+                f(value)
+        return wrapper
+    return decorator
+
+
+def _returnUndoContextManager(get_func):
+    '''
+    Modify the function so that it returns a context manager that,
+    when exiting, restores the previous widget state (saved
+    calling *get_func*)
+    '''
+    def decorator(f):
+        @wraps(f)
+        def wrapper(value):
+
+            class InnerUndoContextManager:
+                def __init__(self, old_value):
+                    self.old_value = old_value
+                def __enter__(self): pass
+                def __exit__(self, *args):
+                    f(self.old_value)
+
+            old_value = get_func()
+            f(value)
+            return InnerUndoContextManager(old_value)
+
+        return wrapper
+    return decorator
+
+
 def _signal_property(widget):
     '''Property that handles the widget's default signal
 
@@ -283,10 +340,11 @@ def _signal_property(widget):
     def getx():
         return widget
 
+    @_alsoAcceptAnotherGui(widget)
     def setx(value):
-        _connect(None, widget, signal_name='default', slot=value)
+        connect(widget, slot=value)
 
-    return _InstanceProperty(getx, setx)
+    return GuiettaProperty(getx, setx)
 
 
 def _text_property(widget):
@@ -296,13 +354,31 @@ def _text_property(widget):
         text = widget.text()
         return _ContextStr(widget, text)
 
+    @_returnUndoContextManager(get_text)
+    @_alsoAcceptAnotherGui(widget)
     def set_text(text):
         if isinstance(widget, SmartQLabel):
             widget.setText(text)
         else:
             widget.setText(str(text))
 
-    return _InstanceProperty(get_text, set_text)
+    return GuiettaProperty(get_text, set_text)
+
+
+def _title_property(widget):
+    '''Property for widgets with a title (QGroupBox)'''
+
+    def get_title():
+        title = widget.title()
+        return _ContextStr(widget, title)
+
+    @_returnUndoContextManager(get_title)
+    @_alsoAcceptAnotherGui(widget)
+    def set_title(title):
+        print('set_title')
+        widget.setTitle(str(title))
+
+    return GuiettaProperty(get_title, set_title)
 
 
 def _value_property(widget, typ):
@@ -310,15 +386,17 @@ def _value_property(widget, typ):
 
     def get_value():
         value = widget.value()
-        if typ == int:
+        if typ is int:
             return _ContextInt(widget, value)
         else:
             return value
 
+    @_returnUndoContextManager(get_value)
+    @_alsoAcceptAnotherGui(widget)
     def set_value(value):
         widget.setValue(typ(value))
 
-    return _InstanceProperty(get_value, set_value)
+    return GuiettaProperty(get_value, set_value)
 
 
 def _readonly_property(widget):
@@ -327,10 +405,11 @@ def _readonly_property(widget):
     def getx():
         return widget
 
+    @_alsoAcceptAnotherGui(widget)
     def setx(x):
         raise AttributeError('This property is read-only')
 
-    return _InstanceProperty(getx, setx)
+    return GuiettaProperty(getx, setx)
 
 
 def _items_property(widget):
@@ -340,12 +419,14 @@ def _items_property(widget):
         items = map(lambda x: x.text(), widget.findItems("*", Qt.MatchWildcard))
         return _ContextList(widget, items)
 
+    @_returnUndoContextManager(get_items)
+    @_alsoAcceptAnotherGui(widget)
     def set_items(lst):
         widget.clear()
         widget.addItems(list(map(str, lst)))  # use list() to support
                                               # PySide v5.9
 
-    return _InstanceProperty(get_items, set_items)
+    return GuiettaProperty(get_items, set_items)
 
 
 def _combobox_property(widget):
@@ -356,12 +437,14 @@ def _combobox_property(widget):
         data = [widget.itemData(i) for i in range(widget.count())]
         return _ContextDict(widget, zip(texts, data))
 
+    @_returnUndoContextManager(get_items)
+    @_alsoAcceptAnotherGui(widget)
     def set_items(dct):
         widget.clear()
         for k, v in dct.items():
             widget.addItem(k, v)
 
-    return _InstanceProperty(get_items, set_items)
+    return GuiettaProperty(get_items, set_items)
 
 
 #########
@@ -410,8 +493,11 @@ class SmartQLabel(QWidget):
     def text(self):
         return self._orig_value
 
+    def __guietta_property__(self):
+        return _text_property(self)
 
-def _fake_property(widget):
+
+def _guietta_property(widget):
     '''Create the instance property corresponding to `widget`'''
 
     if isinstance(widget, QAbstractButton):
@@ -420,6 +506,9 @@ def _fake_property(widget):
     elif isinstance(widget, (QLabel, QLineEdit, SmartQLabel)):
         return _text_property(widget)
 
+    elif isinstance(widget, QGroupBox):
+        return _title_property(widget)
+
     elif isinstance(widget, (QAbstractSlider, QProgressBar)):
         return _value_property(widget, int)
 
@@ -810,7 +899,7 @@ class _ValueSlider(_CombinedWidget):
 
         slider_name = (self.slider, self.name)
 
-        if self.anchor == Qt.AnchorLeft or self.anchor == Qt.AnchorTop:
+        if self.anchor in [Qt.AnchorLeft, Qt.AnchorTop]:
             first, last = self.editbox, slider_name
         else:
             first, last = slider_name, self.editbox
@@ -873,6 +962,7 @@ _default_signals = {QPushButton: 'clicked',
                     QRadioButton: 'toggled',
                     QAbstractSlider: 'valueChanged',
                     QListWidget: 'currentTextChanged',
+                    QGroupBox: 'clicked',
                     QComboBox: 'currentTextChanged'}
 
 
@@ -945,99 +1035,96 @@ def _exception_wrapper(func, mode):
 ############
 # Matplotlib
 
-class MatplotlibWidget:
-    '''Dummy definition to avoid importing matplotlib when it is not used.'''
-    pass
-
-class PyQtGraphPlotWidget:
-    '''Dummy definition to avoid importing pyqtgraph when it is not used.'''
-    pass
-
 @contextlib.contextmanager
 def Ax(widget):
     '''
     Context manager to help drawing on Matplotlib widgets.
 
     Takes care of clearing and redrawing the canvas before and after
-    the inner code block is executed.
+    the inner code block is executed::
 
-    usage:
-        with MatplotlibAx(gui.plot) as ax:
+        with Ax(gui.plot) as ax:
             ax.plot(...)
     '''
-    if not isinstance(widget, MatplotlibWidget):
-        raise TypeError('An instance of MatplotlibWidget is required, '
-                        'got %s instead' % widget.__class__.__name__)
-    ax = widget.ax
+    try:
+        ax = widget.ax
+    except AttributeError as e:
+        raise AttributeError('Class %s does not have an ''ax'' attribute.\n'
+                      'Are you sure that you are referencing the correct plot?'
+                       %  widget.__class__.__name__) from e
     ax.clear()
 
     yield ax
 
+    for k,v in widget.kwargs.items():
+        getattr(widget.ax, k).__call__(v)
     ax.figure.canvas.draw()
 
 
 class M(_DeferredCreationWidget):
-    '''A Matplotlib Canvas widget'''
+    '''A Matplotlib Canvas widget
 
-    def __init__(self, name, width=5, height=3, dpi=100):
+    The **kwargs accepts additional keywords that will become function
+    calls every time the Ax decorator is used. For example, adding the
+    argument `set_ylabel='foo'`, will result in this function call:
+    `ax.set_ylabel('foo')`
+    
+    Creating an object of this class will import the matplotlib module.
+    '''
+    def __init__(self, name, width=5, height=3, dpi=100,
+                       subplots=(1,1), **kwargs):
 
         self._name = name
         self._width = width
         self._height = height
         self._dpi = dpi
+        self._subplots = subplots
+        self._kwargs = kwargs
 
     def create(self, gui):
-        if globals()['MatplotlibWidget'].__name__ == 'MatplotlibWidget':
+        from guietta import guietta_matplotlib
+        widget_class = guietta_matplotlib.MatplotlibWidget
 
-            from matplotlib.figure import Figure
-            from matplotlib.backends.backend_qt5agg import FigureCanvas
+        _default_signals[widget_class] = 'clicked'
 
-            class RealMatplotlibWidget(FigureCanvas):
-
-                clicked = Signal(float, float)
-
-                def __init__(self, width, height, dpi):
-                    figure = Figure(figsize=(width, height), dpi=dpi)
-                    self.ax = figure.add_subplot(111)
-                    super().__init__(figure)
-                    figure.canvas.mpl_connect('button_press_event',
-                                              self._on_button_press)
-
-                def _on_button_press(self, event):
-                    self.clicked.emit(event.xdata, event.ydata)
-
-            globals()['MatplotlibWidget'] = RealMatplotlibWidget
-            _default_signals[RealMatplotlibWidget] = 'clicked'
-
-
-        widget = MatplotlibWidget(self._width, self._height, self._dpi)
+        widget = widget_class(self._width, self._height, self._dpi,
+                              self._subplots, **self._kwargs)
         return (widget, self._name)
 
 
 class PG(_DeferredCreationWidget):
-    '''A pyqtgraph PlotWidget'''
+    '''A pyqtgraph PlotWidget.
+    
+    Creating an object of this class will import the pyqtgraph module.'''
+
+    _pyqtgraph_imported = False
+    
+    def __init__(self, name, **kwargs):
 
-    def __init__(self, name):
         self._name = name
+        self._kwargs = kwargs
 
     def create(self, gui):
-        if globals()['PyQtGraphPlotWidget'].__name__ == 'PyQtGraphPlotWidget':
-
-            import pyqtgraph
-            if pyqtgraph.__version__ < '0.11.0':
-                raise Exception('Minimum version for pyqtgraph is 0.11.0,'
-                                'you have '+pyqtgraph.__version__)
-
-            class RealPyQtGraphPlotWidget(pyqtgraph.PlotWidget):
-                def __init__(self):
-                    super().__init__()
-
-            globals()['PyQtGraphPlotWidget'] = RealPyQtGraphPlotWidget
-
-        widget = PyQtGraphPlotWidget()
+        from guietta import guietta_pyqtgraph
+        widget =  guietta_pyqtgraph.PyQtGraphPlotWidget(**self._kwargs)
         return (widget, self._name)
       
 
+class PGI(_DeferredCreationWidget):
+    '''A pyqtgraph ImageView.
+    
+      Creating an object of this class will import the pyqtgraph module.'''
+
+    def __init__(self, name, **kwargs):
+        self._name = name
+        self._kwargs = kwargs
+
+    def create(self, gui):
+        from guietta import guietta_pyqtgraph
+        widget =  guietta_pyqtgraph.PyQtGraphImageView(**self._kwargs)
+        return (widget, self._name)
+
+
 #####################
 # Stdout redirection
 
@@ -1073,9 +1160,12 @@ class StdoutLog(QPlainTextEdit):
 
 # Some helper functions
 
-def _normalize(x):
-    '''Return x without all special characters. Keep only a-zA-Z0-9 and _'''
-    return ''.join(c for c in x if c.isalnum() or c == '_')
+def normalized(name):
+    '''Returns the given *name* without any special characters or spaces.
+
+    Only a-zA-Z0-9 and _ will be kept
+    '''
+    return ''.join(c for c in name if c.isalnum() or c == '_')
 
 
 def _bound_method(method, to_whom):
@@ -1104,7 +1194,7 @@ def _create_default_widgets(x):
 
     # Also recurse into (WidgetClass, 'name')
     if (
-        type(x) == tuple
+        type(x) is tuple
         and (len(x) == 2)
         and isinstance(x[1], str)
     ):
@@ -1151,6 +1241,38 @@ def _process_slots(x):
     else:
         raise ValueError('Element %s is not a valid slot assignment' % x)
 
+def _process_font(x):
+    '''Normalize font assignments.
+
+    *x* must be either a QFont instance, a string with a font
+    family name (.e.g 'Helvetica'), or a sequence with the QFont
+    constructor elements: family string, point size, weight, and italic,
+    int that order. All except the family string are optional, point size
+    and weight ar integers, and italic is a boolean True/False. Use -1
+    for arguments that must be left at their default value while specifying
+    a subsequent argument. All these specifications are valid:
+
+        - QFont('helvetica', pointSize=12)
+        - 'helvetica'
+        - ['Arial', -1, -1, False]
+        - ('helvetica', 12, 1, True)
+    '''
+    if isinstance(x, QFont):
+        return x
+
+    if x in _specials:
+        return x
+
+    elif isinstance(x, str):
+        return QFont(x)
+
+    elif _sequence(x):
+        try:
+            return QFont(*x)
+        except Exception as e:
+            raise ValueError('%s rejected by QFont constructor' % x) from e
+    else:
+        raise TypeError('Element %s is not a valid font specification' % x)
 
 def _check_string(x):
     if not isinstance(x, str) and x not in _specials:
@@ -1374,6 +1496,8 @@ def splash(text,
            image=None):
     '''Display and return a splash screen.
 
+    This function displays a splashscreen and returns a QSplashScreen instance.
+
     The splashscreen must be closed with close() or finish(gui.window()).
     Alternatively, it will close when the user clicks on it.
     '''
@@ -1394,15 +1518,13 @@ def splash(text,
     return splash
 
 
-def _connect(gui_obj, widget, signal_name, slot):
+def connect(widget, signal_name='default', slot=None):
 
-    # If called without a gui_obj, look it up using the widget attributes.
-    if gui_obj is None:
-        if hasattr(widget, '_gui'):
-            gui_obj = widget._gui
-        else:
-            raise TypeError('Widget %s does not seem to belong to any '
-                            'Gui object' % widget.__class__.__name__)
+    if hasattr(widget, '_gui'):
+        gui = widget._gui
+    else:
+        raise TypeError('Widget %s does not seem to belong to any '
+                        'Gui object' % widget.__class__.__name__)
 
     if signal_name == 'default':
         try:
@@ -1418,14 +1540,14 @@ def _connect(gui_obj, widget, signal_name, slot):
 
     # Custom signal with default handler for get()
     if slot is None:
-        slot = functools.partial(gui_obj._event_handler, signal, widget)
+        slot = functools.partial(gui._event_handler, signal, widget)
 
-    if _bound_method(slot, to_whom=gui_obj):
+    if _bound_method(slot, to_whom=gui):
         use_slot = slot
     else:
-        use_slot = functools.partial(slot, gui_obj)
+        use_slot = functools.partial(slot, gui)
 
-    signal.connect(_exception_wrapper(use_slot, gui_obj._exception_mode))
+    signal.connect(_exception_wrapper(use_slot, gui._exception_mode))
 
 
 class Gui:
@@ -1449,11 +1571,12 @@ class Gui:
                                create_properties=True,
                                exceptions=Exceptions.POPUP,
                                persistence=PERSISTENT,
-                               title=''):
+                               title='',
+                               font=None):
 
         # This line must be the first one in this method otherwise
         # __setattr__ does not work.
-        self.__dict__['_fake_properties'] = {}
+        self.__dict__['_guietta_properties'] = {}
 
         self.userdata = SimpleNamespace()
 
@@ -1463,8 +1586,10 @@ class Gui:
         self._layout = QGridLayout()
         self._widgets = {}                # widgets by name
         self._counter = defaultdict(int)  # widgets counter
+        self._original_names = {}         # Reverse widget name lookup
         self._window = None
         self._app = QApplication.instance()
+        self._font = font
 
         self._get_handler = False   # These three for the get() method
         self._event_queue = queue.Queue()
@@ -1533,6 +1658,8 @@ class Gui:
                         colspan += 1
 
                 widget, name = self._get_widget_and_name(element)
+                if hasattr(widget, '_gui'):
+                    raise Exception("Widget %s already has a '_gui' attribute" % name)
                 widget._gui = self
                 ContextMixIn.convert_object(widget)
                 self._layout.addWidget(widget, i, j, rowspan, colspan)
@@ -1545,36 +1672,54 @@ class Gui:
 
                 done.add(element)
 
-        self.align_fake_properties()
+        self._align_guietta_properties()
         self.title(title)
 
-    def align_fake_properties(self):
+    def _align_guietta_properties(self):
         '''Make sure that any and all widgets have a property'''
 
-        self._fake_properties.clear()
+        self._guietta_properties.clear()
         if self._create_properties:
             for name, widget in self._widgets.items():
-                self._fake_properties[name] = _fake_property(widget)
+                if hasattr(widget, '__guietta_property__'):
+                    prop = GuiettaProperty(*widget.__guietta_property__())
+                else:
+                    prop = _guietta_property(widget)
+                self._guietta_properties[name] = prop
 
     @property
     def widgets(self):
         '''Read-only property with the widgets dictionary'''
         return self._widgets
 
-    def __getattr__(self, name):
-        '''Use fake_properties to emulate properties on this instance'''
+    @property
+    def names(self):
+        '''Read-only property with the normalized -> original mapping'''
+        return self._original_names
 
-        if name in self.__dict__['_fake_properties']:
-            return self.__dict__['_fake_properties'][name].get()
+    def proxy(self, name):
+        '''Returns the *guietta property* for the a (normalized) widget name.
+
+        A guietta property is an instance of the *GuiettaProperty* namedtuple,
+        with two members: get() and set()
+        '''
+        name = normalized(name)
+        return self.__dict__['_guietta_properties'][name]
+
+    def __getattr__(self, name):
+        '''Use guietta_properties to emulate properties on this instance'''
+
+        if name in self.__dict__['_guietta_properties']:
+            return self.__dict__['_guietta_properties'][name].get()
 
         # Default behaviour
         raise AttributeError(name)
 
     def __setattr__(self, name, value):
-        '''Use fake_properties to emulate properties on this instance'''
+        '''Use guietta_properties to emulate properties on this instance'''
 
-        if name in self.__dict__['_fake_properties']:
-            self.__dict__['_fake_properties'][name].set(value)
+        if name in self.__dict__['_guietta_properties']:
+            self.__dict__['_guietta_properties'][name].set(value)
             return
 
         # Default behaviour
@@ -1586,28 +1731,33 @@ class Gui:
         Several possibilities:
             - (widget, 'name')  - type checks should have already been
                                   performed before, hopefully
-            - widget            - if widget defines text(), use that as
-                                  its name, otherwise use the class name
+            - widget            - if widget defines text() or title(), use
+                                  that as the name, otherwise
+                                  use the class name
 
         - remove special characters, only leave a-zA-Z0-9
         - auto-number duplicate names.
         '''
         if isinstance(element, tuple):
-            widget, name = element
+            widget, original_name = element
         else:
             widget = element
             if hasattr(widget, 'text'):
-                name = widget.text()
+                original_name = widget.text()
+            elif hasattr(widget, 'title'):
+                original_name = widget.title()
             else:
-                name = widget.__class__.__name__
+                original_name = widget.__class__.__name__
 
-        name = _normalize(name)
+        name = normalized(original_name)
 
         # If the name is a duplicate, auto-number it starting with 2.
         self._counter[name] += 1
         if self._counter[name] > 1:
             name = name + str(self._counter[name])
 
+        self._original_names[name] = original_name
+
         return widget, name
 
     def row_stretch(self, *lists):
@@ -1673,7 +1823,36 @@ class Gui:
         for i, j, pair in rows.enumerate():
             item = self[i,j]
             signal_name, slot = pair
-            _connect(self, item, signal_name, slot)
+            connect(item, signal_name, slot)
+
+    def fonts(self, *lists):
+        '''Defines the fonts used for each GUI widget.
+
+        Arguments are lists as in the initializer. It is allowed to define
+        just one or only a few rows in this method, if for example
+        the last rows do not contain widgets whose fonts need not
+        to be modified.
+
+        Every element is either a QFont instance, a string with a font
+        family name (.e.g 'Helvetica'), or a tuple with the QFont
+        constructor elements: family string, point size, weight, and italic,
+        int that order. All except the family string are optional, point size
+        and weight ar integers, and italic is a boolean True/False. All these
+        specifications are valid:
+
+            - QFont('helvetica', pointSize=12)
+            - 'helvetica'
+            - ('helvetica', 12, 1, True)
+
+        Use _ for widgets that do not need their fonts to be changed.
+        '''
+        rows = Rows(lists)
+        rows.check_same(self._rows, allow_less_rows=True)
+
+        rows.map_in_place(_process_font)
+
+        for i, j, font_spec in rows.enumerate():
+            self[i,j].setFont(font_spec)
 
     def rename(self, *lists):
         '''Overrides the default widget names.
@@ -1700,7 +1879,7 @@ class Gui:
             self._widgets[new_name] = self._widgets[old_name]
             del self._widgets[old_name]
 
-        self.align_fake_properties()
+        self._align_guietta_properties()
 
     def __getitem__(self, name):
         '''Widget by coordinates [row,col]'''
@@ -1719,8 +1898,13 @@ class Gui:
             self._window = QWidget()
             self._window.setLayout(self._layout)
             self._window.closeEvent = self._close_handler
+            if self._font is not None:
+                self._window.setFont(self._font)
         return self._window
 
+    def font(self):
+        return self.window().font()
+
     def _close_handler(self, event):
         _remove_from_persistence_list(self)
 
@@ -1856,8 +2040,9 @@ class Gui:
 
         if not callable(func):
             raise TypeError('func must be a callable')
-        if not callable(callback):
-            raise TypeError('callback must be a callable')
+        if callback is not None:
+            if not callable(callback):
+                raise TypeError('callback must be a callable')
 
         app = QApplication.instance()
         app.customEvent = _customEvent
@@ -1911,7 +2096,7 @@ class Gui:
             if widget_name in self.widgets:
                 try:
                     widget = self.widgets[widget_name]
-                    _connect(self, widget, 'default', func)
+                    connect(widget, slot=func)
 
                 except ValueError:
                     # No default signal defined
